#include "hsv.h"
#include "math.h"

#define max(a, b) ((a > b) ? a : b)
#define min(a, b) ((a < b) ? a : b)

float floatmod(float a, float b) {
  return a - b*floor(a/b);
}

void rgb_to_hsv(vec3 rgb, double hsv[3]) {
  if(rgb[0] == 0 && rgb[1] == 0 && rgb[2] == 0) {
    hsv[0] = 0;
    hsv[1] = 0;
    hsv[2] = 0;
    return;
  }

  hsv[2] = max(rgb[0], max(rgb[1], rgb[2]));
  float M = min(rgb[0], min(rgb[1], rgb[2]));
  float C = hsv[2] - M;
  hsv[1] = C / hsv[2];
  float X;

  if(hsv[2] == rgb[0] && M == rgb[2]) {
    X = rgb[1] - M;
    hsv[0] = (X/C + 0)/6;
  } else if(hsv[2] == rgb[1] && M == rgb[2]) {
    X = rgb[0] - M;
    hsv[0] = (2 - X/C)/6;
  } else if(hsv[2] == rgb[1] && M == rgb[0]) {
    X = rgb[2] - M;
    hsv[0] = (X/C + 2)/6;
  } else if(hsv[2] == rgb[2] && M == rgb[0]) {
    X = rgb[1] - M;
    hsv[0] = (4 - X/C)/6;
  } else if(hsv[2] == rgb[2] && M == rgb[1]) {
    X = rgb[0] - M;
    hsv[0] = (X/C + 4)/6;
  } else if(hsv[2] == rgb[0] && M == rgb[1]) {
    X = rgb[2] - M;
    hsv[0] = (6 - X/C)/6;
  }
}

void hsv_to_rgb(double hsv[3], vec3 rgb) {
  float C = hsv[1] * hsv[2];
  float H = hsv[0]*6;
  float X = C * (1 - fabs(floatmod(H, 2) - 1));
  vec4 temp = {0, 0, 0, 0};
  if(0 <= H && H <= 1) {
    temp[0] = C;
    temp[1] = X;
  } else if(1 <= H && H <= 2) {
    temp[0] = X;
    temp[1] = C;
  } else if(2 <= H && H <= 3) {
    temp[1] = C;
    temp[2] = X;
  } else if(3 <= H && H <= 4) {
    temp[1] = X;
    temp[2] = C;
  } else if(4 <= H && H <= 5) {
    temp[0] = X;
    temp[2] = C;
  } else if(5 <= H && H <= 6) {
    temp[0] = C;
    temp[2] = X;
  }

  float m = hsv[2] - C;
   rgb[0] = temp[0] + m;
   rgb[1] = temp[1] + m;
   rgb[2] = temp[2] + m;
}
